from meiga import Result, Error

from petisco import Event


def send_mail_handler(event: Event) -> Result[bool, Error]:
    pass


def send_sms_handler(event: Event) -> Result[bool, Error]:
    pass
